#!/bin/bash
set -euo pipefail
WD=$(mktemp -d)
cleanup (){
	rm -rf "${WD}"
}
trap cleanup EXIT

DELIVERED=false
SENDMAIL="/usr/sbin/sendmail"
if (($DEBUG != 0)); then
	SENDMAIL="$DEBUGMAIL"
fi

cat - | formail -kb \
	-X "MIME-Version:" \
	-X "Content-Type:" \
	-X "Subject:" \
	-X "Date:" \
> "${WD}/mail"
subject="$(formail -cx "Subject:" < "${WD}/mail")"
if [ "${subject/\[${SUBJECT}\]/}" == "${subject}" ]; then
	subject="[${SUBJECT}] ${subject## }"
fi
formail -I "Subject: ${subject}" < "${WD}/mail" | tail -n+2 > "${WD}/mail.new"
mv "${WD}"/mail{.new,}

declare -A SUBSCRIBED
declare -a actions
update(){
	da="${actions[$1-1]:0:1}"
	db="${2:0:1}"
	alias="${actions[$1-1]#?}"
	email="${2#?}"
	[ "$da" == "$db" ] || exit 1 # corrupt list
	if [ "$da" == "+" ]; then
		SUBSCRIBED["$email"]="$alias"
	else
		SUBSCRIBED["$email"]=""
	fi
}

exec 4<> "${SLIST}.txt"
flock 4
mapfile -t -c 2 -C update -u 4 actions
for email in "${!SUBSCRIBED[@]}"; do
	if [ -z "${SUBSCRIBED["$email"]}" ]; then
		 continue
	fi
	echo "+${SUBSCRIBED["$email"]}"
	echo "+$email"
done > "${SLIST}".txt
flock -u 4

for email in "${!SUBSCRIBED[@]}"; do
	if [ -z "${SUBSCRIBED["$email"]}" ] || [ "${email,,}" == "${NOFORWARD,,}" ]; then
		continue
	fi
	if [ "$TO" != "" ] && [ "$email" != "$TO" ]; then
		continue
	fi
	echo "Forwarding mail to: ${SUBSCRIBED["$email"]} <${email}>"
	"$SENDMAIL" -F "${FROM}" -f "${MAILGROUP}" "${SUBSCRIBED["$email"]} <${email}>" < "${WD}/mail"
	DELIVERED=true
done
if ! $DELIVERED; then
	"$DEBUGMAIL" undelivered
	exit "$UNHANDLED"
fi
